* MakPalet.prg
*                                              by Ken Chan (HazMatZak @ BORBBS)
*
* Palette set code generator for use with COLORCTL.bin by George E. McMullen
*
*       This program creates and adds to an "external procedure" file
*       named PALETTE_.prg.  You do not have to SET PROCEDURE to
*       utilize this file.  Simply
*
*               do Palette_<palette ID>
*       e.g.
*               do Palette_1
*
*       where <palette ID> is a single letter or digit.
*
*       The advantages of this external procedure file over using
*       multiple .mem files for different palettes are:
*       (a) The resulting .dbo is smaller than the .mem file, and
*           considering cluster granularity, one .dbo with lots of
*           procedures is much smaller than lots of .mem files.
*       (b) The resulting code is faster.
*       (c) Portability -- no need to cart around lots of .mem files.
*
*       To create different palettes, use SETCLR.prg (part of the
*       COLORCTL.zip package) to change the palette, save the
*       palette values to the COLOR.mem file, then run this program.
*
*       You will be prompted for a palette ID.  MakPalet will
*       copy the COLOR.mem file to a new .mem file with the palette
*       ID, and then check to see if the ID is already being
*       used in PALETTE_.prg, if such a file exists.  You also have
*       the opportunity to add a one-line comment.
*
*       If this is a new palette or you opt to replace the old one,
*       the code will be added to PALETTE_.prg, and then the file
*       will be COMPILEd.
*
* Two palettes will be generated automatically the first time you
* add a new palette:
*
*       Palette_1 = Standard VGA
*       Palette_0 = all black
*
* Comments and suggestions welcome.
*
* 12/17/91 v1.1
*

wait "Palette ID (1 letter or digit): " to cPalID
accept "One line comment for this palette: " to cComment
cPalID = upper (cPalID)
cPalMem = "PALETT_" + cPalID + ".mem"
if .not. file (cPalMem)
  if file ("COLOR.mem")
    wait "Copy current COLOR.mem to " + cPalMem + "? (Y/n) " to cYN
    if lower (cYN) = "n"
      return
    endif
    copy file COLOR.mem to (cPalMem)
  else
    ? chr (7) + "### Error: unable to find " + cPalMem + " or COLOR.mem"
    return
  endif
endif
if file ("PALETTE_.prg")
  ? "Checking for old palette..."
  cDbf = MakDbf ()
  use (cDBF)
  append blank
  append memo PRG_FILE from PALETTE_.prg overwrite
  if mline (PRG_FILE, 7) # "*   do Palette_<palette ID>"
    ? chr (7) + "### Error: existing PALETTE_.prg not associated with this utility."
    ? "Rename existing file and restart."
    do CleanUp
    return
  endif
  if "PROCEDURE Palette_" + cPalID $ PRG_FILE
    wait "Old palette " + cPalID + " exists.  Replace it? (y/N) " to cYN
    if lower (cYN) = "y"
      lSafety_On = set ("safety") = "ON"
      set safety off
      zap
      if lSafety_On
        set safety on
      endif
      ? "  - Reading .prg file"
      append from PALETTE_.prg type sdf
      *-- Add a blank line at end of "file"
      append blank
      ? "  - Locating procedure"
      locate for PRG_LINE = "PROCEDURE Palette_" + cPalID
      if found ()
        ? "  - Deleting procedure code"
        delete while (PRG_LINE) # "RETURN"
        delete
        if "" = trim (PRG_LINE) .and. .not. eof ()
          delete
        endif
        ? "  - Saving changes"
        if file ("PALETTE_.bak")
          erase PALETTE_.bak
        endif
        rename PALETTE_.prg to PALETTE_.bak
        set alternate to PALETTE_.prg
        go top
        set alternate on
        set console off
        ??trim (PRG_LINE)
        skip
        scan rest for .not. deleted ()
          ? trim (PRG_LINE)
        endscan
        set console on
        set alternate off
        close alternate
      else
        ? chr (7) + "### Abend: unable to relocate procedure"
      endif
    else
      do CleanUp
      return
    endif && replace old procedure
  endif && procedure exists
  do CleanUp
else
  ? "Creating new PALETTE_.prg with Default and Black palettes..."
  set alternate to PALETTE_.prg
  set alternate on
  set console off
  ??"* Palette_.prg"
  text       && --------------------------------------------------------
*
* Set VGA color palettes.
* Requires COLORCTL.bin by George E. McMullen.
*
* External procedure file -- all calls of the form
*   do Palette_<palette ID>
* e.g.
*   do Palette_1
*
* SET PROCEDURE is neither required nor desired!
*
* Code generated by MakPalet.prg by Ken Chan (HazMatZak @ BORBBS)
*
  endtext    && --------------------------------------------------------
  ? "* " + dtoc (date ())
  text       && --------------------------------------------------------
*

PROCEDURE Palette_1
  * Standard VGA
  * 12/13/91
  load COLORCTL
  *-- This will reset all colors to standard
  call COLORCTL with "/R"
  *-- For reference, those values are
  note COLORCTL with "/S","Index= 0","Red= 0","Green= 0","Blue= 0"
  note COLORCTL with "/S","Index= 1","Red= 0","Green= 0","Blue=42"
  note COLORCTL with "/S","Index= 2","Red= 0","Green=42","Blue= 0"
  note COLORCTL with "/S","Index= 3","Red= 0","Green=42","Blue=42"
  note COLORCTL with "/S","Index= 4","Red=42","Green= 0","Blue= 0"
  note COLORCTL with "/S","Index= 5","Red=42","Green= 0","Blue=42"
  note COLORCTL with "/S","Index= 6","Red=42","Green=21","Blue= 0"
  note COLORCTL with "/S","Index= 7","Red=42","Green=42","Blue=42"
  note COLORCTL with "/S","Index= 8","Red=21","Green=21","Blue=21"
  note COLORCTL with "/S","Index= 9","Red=21","Green=21","Blue=63"
  note COLORCTL with "/S","Index=10","Red=21","Green=63","Blue=21"
  note COLORCTL with "/S","Index=11","Red=21","Green=63","Blue=63"
  note COLORCTL with "/S","Index=12","Red=63","Green=21","Blue=21"
  note COLORCTL with "/S","Index=13","Red=63","Green=21","Blue=63"
  note COLORCTL with "/S","Index=14","Red=63","Green=63","Blue=21"
  note COLORCTL with "/S","Index=15","Red=63","Green=63","Blue=63"
  release module COLORCTL
RETURN

PROCEDURE Palette_0
  * All black -- "turn off" screen
  * 12/13/91
  load COLORCTL
  call COLORCTL with "/S","Index= 0","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 1","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 2","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 3","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 4","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 5","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 6","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 7","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 8","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index= 9","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=10","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=11","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=12","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=13","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=14","Red= 0","Green= 0","Blue= 0"
  call COLORCTL with "/S","Index=15","Red= 0","Green= 0","Blue= 0"
  release module COLORCTL
RETURN

  endtext    && --------------------------------------------------------
  set console on
  set alternate off
  close alternate
endif && .prg exists
? "Writing palette values for " + cPalID + "..."
set alternate to PALETTE_.prg additive
set alternate on
set console off
? "PROCEDURE Palette_" + cPalID
? "  * " + cComment
? "  * " + dtoc (date ())
? "  load COLORCTL"
restore from (cPalMem) additive
nIndex = 1
do while nIndex <= 16
  ? '  call COLORCTL with "/S",'
  ??'"Index='+str(gl_colors[nIndex,1],2)+'",'
  ??'"Red='  +str(gl_colors[nIndex,2],2)+'",'
  ??'"Green='+str(gl_colors[nIndex,3],2)+'",'
  ??'"Blue=' +str(gl_colors[nIndex,4],2)+'"'
  nIndex = nIndex + 1
enddo
? "  release module COLORCTL"
? "RETURN"
?
set console on
set alternate off
close alternate
? "Compiling PALETTE_.prg..."
compile PALETTE_
? "--- Finished!"
return
*-- EoP: MakPalet.prg



PROCEDURE CleanUp
  *-- Close and erase temp files
  use
  erase (cDbf)
  erase (stuff (cDbf, at (".dbf", cDbf), 4, ".dbt"))
RETURN



FUNCTION MakDbf
  *----------------------------------------------------------------------------
  *-- Programmer..: HazMatZak
  *-- Date........: 12/13/91
  *-- Notes.......: Creates temp .dbf and .dbt for MakPalet.prg from scratch.
  *                 See Aug 1990 TechNotes "Made to Order" by Roland Bouchereau
  *-- Written for.: dBASE IV, 1.1
  *-- Rev. History: 12/13/1991 1.0
  *-- Calls.......: <none>
  *-- Called by...: MakPalet.prg
  *-- Usage.......: MakDbf ()
  *-- Example.....: MakDbf ()
  *-- Returns.....: Temp file name with .dbf extension
  *-- Parameters..: <none>
  *----------------------------------------------------------------------------
  private cFileName, _pdriver, _pform
  cFileName = "TMP" + str (rand () * 1000, 3) + "_P.dbf"
  do while file (cFileName)
    cFileName = "TMP" + str (rand () * 1000, 3) + "_P.dbf"
  enddo
  *-- Set drivers to avoid codes
  _pdriver = "ASCII.pr2"
  _pform = ""
  *-- Create .dbf
  set printer to file (cFileName)
  *-- First byte indicates .dbf with memo field
  ??? "{139}"
  *-- Date of last update
  ??? chr (year (date ()) - 1900) + chr (month (date ())) + chr (day (date ()))
  *-- No records
  ??? "{0}{0}{0}{0}"
  *-- Number of bytes in header
  ??? "{97}{0}"
  *-- Number of bytes in each record
  ??? "{9}{1}"
  *-- Fill rest of header
  ??? replicate ("{0}", 20)
  *-- Field structure
  ??? "PRG_LINE{0}{0}{0}C{0}{0}{0}{0}{254}" + replicate ("{0}", 15)
  ??? "PRG_FILE{0}{0}{0}M{0}{0}{0}{0}{10}"  + replicate ("{0}", 15)
  *-- Header terminator
  ??? "{13}"
  *-- Close .dbf
  set printer to
  *-- Create .dbt
  set printer to file (stuff (cFileName, at (".dbf", cFileName), 4, ".dbt"))
  *-- Memo file
  ??? "{1}{0}{0}{0}{0}{0}{0}{0}"
  ??? left (cFileName, 8)
  ??? "{0}{0}{2}{1}{0}{2}{0}{0}"
  *-- Close .dbt
  set printer to
RETURN cFileName
